/////////////////////////////////////////////////////////////////////////////////
// game.gs
// Users write here basic functions specific for their game. Also check the gamedef.gs
// Advanced users may need to adjust other files as well.
//
// Here are the callback functions that usually stay in this file:
//
// Initialization callbacks:
// RoomsSetNames, ObjectsSetNames, RoomSetCustomText, SaveUserData, LoadUserData, BeginNewGame
//
// Rooms interaction callbacks:
// UpdateRoom_RX_RY()			- called by HandlerGameUpdate while player is in room RX,RY (non-latent)
// AfterRoom_RX_RY()			- called by HandlerGameAfterUpdate while player is in room RX,RY (non-latent)
// OpenRoom_RX_RY()				- called by HandlerRoomOpen when room RX,RY is opened (non-latent)
// CloseRoom_RX_RY()			- called by HandlerRoomClose when room RX,RY is closed (non-latent)
// OutRoom_RX_RY()				- called by HandlerRoomOut when player wants to exit room RX,RY (can reposition player here) (non-latent)
//
// Objects interaction callbacks:
// PickupObject_ID()			- called when picking up item ID (latent)
// DropObject_ID()				- called when droping item ID (latent)
// ActionObject_ID()			- called when player hits ACTION key on object ID (latent)
// UseObject_ID( idx )			- called when player wants to use object idx (from inventory) over the ID object (from the map) (latent)
// CollideObject_ID_MODE()		- called when player collides with object ID in mode MODE (0=exit from collision, 1=just entered collision, 2=continuing to collide) (latent)
//
// Player callbacks:
// PlayerDeathMessage( death ) 	- called by PlayerLoseLife and sould just return the death message text. return "" for no death message box (latent)
// RespawnPlayer_DEATH()		- called by PlayerLoseLife for custom death respawns. DEATH is the value of player's P_DEATH property
//
// ID, RX, RY, MODE are to pe replaced with coresponding numbers
//
/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
// Sets room names (non-latent)
// By default it loads them from the dizzy.nam file, using the RoomsLoadNames() function.
// You can also set the names for each room, by hand, with RoomSetName().
// Ex: RoomSetName( 1,1, "PRESS ACTION TO START" );
/////////////////////////////////////////////////////////////////////////////////
func RoomsSetNames()
{
	RoomsLoadNames(ROOM_NAMESFILE);
}

/////////////////////////////////////////////////////////////////////////////////
// Sets names to object items (non-latent)
// All items tht you might pick up and view in the inventory, must have names set.
// Use the ID you specified for the object in the map editor, to find the object.
// See ObjSetName() and ObjFind().
// Ex: ObjSetName(ObjFind(100),"BUCKET");
/////////////////////////////////////////////////////////////////////////////////
func ObjectsSetNames()
{
	// ...
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int; rx; room x coordinate
// IN: int; ry; room y coordinate
// IN: int; idx; room custom text index (0-3)
// IN: ref; strref; string reference to the custom text
// Sets custom texts for each room (non-latent)
// There are 4 custom texts per room, that can be set from the map editor.
// They are saved by default in the ROOM_TEXTSFILE file (dizzy.rt)
// Each game receives these texts through this callback when the map is loaded
// and it can interpret them as it wants, for example to fill custom structures
// per each room, like ambient sounds, or any other things.
// Only non-empty strings are saved from the editor.
// This function uses string reference for speed.
/////////////////////////////////////////////////////////////////////////////////
func RoomSetCustomText( rx, ry, idx, refstr )
{
	// println("RCT: ",rx,",",ry,",",idx," = ",(*refstr));
	// ...
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int; death; cause of death
// Returns the player death message. Called by PlayerLoseLife().
// Declare more death defines in gamedef.gs (like DEATH_INFIRE, or DEATH_BATS)
// and set them to hurt and kill objects or just set them in the player's 
// P_DEATH property, then return specific messages in this callback, 
// for each cacuse of death .
/////////////////////////////////////////////////////////////////////////////////
func PlayerDeathMessage( death )
{
	if(death==-1)					return "";
	// ...
	return "YOU HAVE DIED!";		// default
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int; file; file handler
// OUT: int; return 0 if something has failed, or 1 if operation was successful
// Saves additional user data.
// Users can save here the additional data (like global GS9 variables) they might need to place in the saved game file.
// Called from SaveGame(). See LoadUserData().
// Ex: if(!gs_filewriteint(g_myvariable,file)) return 0;
/////////////////////////////////////////////////////////////////////////////////
func SaveUserData( file )
{
	// ...
	return 1;
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int; file; file handler
// OUT: int; return 0 if something has failed, or 1 if operation was successful
// Load additional user data.
// Users can load here the additional data (like global GS9 variables) they saved before.
// They can also set various things, that depends on the just loaded data.
// For example, since the room's or object's names are not stored in the saved game, 
// if such a name is changed as the result of a solved puzzle, when the game is loaded, 
// it must be changed again, accordingly to the status of the puzzle.
// Called from LoadGame(). See SaveUserData(). 
// Ex: if(!gs_filereadint(&g_myvariable, file)) return 0;
/////////////////////////////////////////////////////////////////////////////////
func LoadUserData( file )
{
	// ...
	return 1;
}

/////////////////////////////////////////////////////////////////////////////////
// This function is called from MainMenu when a new game begins.
// Users must write here whatever they need their game to do, when it's started.
// For example here can be placed or called an intro sequence.
// In the end, the game must be unpaused, the player must be positioned where he must start, game music can be played, etc.
// In the Default Template this also opens a "Hello World!" message.
// Latent function.
/////////////////////////////////////////////////////////////////////////////////
func upload_materials()
{
	

	mapw = GameGet(G_MAPW) ;
	maph = GameGet(G_MAPH) ;
	roomw=GameGet(G_ROOMW);
	roomh=GameGet(G_ROOMH);
	ff = gs_fileopen("mat.0",1);

	mat = tab(17);

	
	
	for(a=0;a<17;a++)
	{
		mat[a]= tab(30);
	}
	
	
	//-------------------------------
	//метериалы
	//-------------------------------
	shift=0;
	cnt=0;
	shifts=tab(maph*mapw+1);
	mats=tab(16*1024);
	curshift=0;
	shifts[curshift]=shift;
	curshift++;
	curmat=0;
	curmatcnt=-1;
	
	println(GameGet(G_MAPW));
	for(a=0;a<maph;a++)
	{
		for(b=0;b<mapw;b++)
		{
			PlayerSetPos( b*8*30+64, a*8*17+64 );
			WaitFramesClearKeys(1);
			
			for(d=0;d<17;d++)
			{
				for(c=0;c<30;c++)
				{
					
					mat[d][c]=0;
					mt=MaterialRead( c*8, d*8, 8, 8 );
					for(aa=0;aa<16;aa++)
					{
						mt1=mt;
						mt1=mt1&1;
						if(mt1==1)
						{
							mat[d][c]=aa;
						}
						mt=mt>>1;
					}
					
				}
				
			}
			
			//-------------------------------------
			for(d=0;d<17;d++)
			{
				for(c=0;c<30;c++)
				{
					print(mat[d][c]);
					if(curmatcnt==-1)
					{
						curmat=mat[d][c];
						curmatcnt=1;
					}
					else
					{
						if(curmat==mat[d][c])
						{
							curmatcnt=curmatcnt+1;
						}
						else
						{
							mats[cnt]=curmatcnt;
							shift++;
							shift++;
							cnt++;
							mats[cnt]=curmat;
							shift++;
							cnt++;
							curmatcnt=1;
							curmat=mat[d][c];
						}
					}
				}
				println("|||");
			}
			println(shifts[curshift-1]+(mapw*maph*2));
			println("___");
			mats[cnt]=curmatcnt;
			shift++;
			shift++;
			cnt++;
			mats[cnt]=curmat;
			shift++;
			cnt++;
			curmatcnt=-1;
			curmat=0;
			
			
			shifts[curshift]=shift;
			curshift++;
		}
	}
	curshift--;
	
	for(a=0;a<curshift;a++)
	{
		gs_filewriteint(shifts[a]+(mapw*maph*2),ff);
		println(shifts[a]+(mapw*maph*2));
		gs_fileseek(ff,-2,1);
	}
	for(a=0;a<cnt;a++)
	{
		gs_filewriteint(mats[a],ff);
		gs_fileseek(ff,-2,1);
		a++;
		gs_filewriteint(mats[a],ff);
		gs_fileseek(ff,-3,1);
	}
	gs_fileclose(ff);
	println("*********>>>");
	
	GameCommand(CMD_EXIT);
}
func write_color(color,f)
{
	if(color==0xffffffff )//not colored
	{
		gs_filewriteint(10,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(10,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff8c8c8c)//grey
	{
		gs_filewriteint(7,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff505050)//dark grey
	{
		gs_filewriteint(7,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff8c0000)//red bright
	{
		gs_filewriteint(2,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff500000)//red
	{
		gs_filewriteint(2,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff8c008c)//cyan bright
	{
		gs_filewriteint(3,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff500050)//cyan
	{
		gs_filewriteint(3,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff008c00)//green bright
	{
		gs_filewriteint(4,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff005000)//green
	{
		gs_filewriteint(4,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff008c8c)//lightblue bright
	{
		gs_filewriteint(5,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff005050)//lighblue
	{
		gs_filewriteint(5,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff8c8c00)//yello bright
	{
		gs_filewriteint(6,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xffffff00)//yello bright
	{
		gs_filewriteint(6,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff505000)//yello
	{
		gs_filewriteint(6,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff000000)//black
	{
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}
	if(color==0xff00008c)//blue bright
	{
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
	}

	if(color==0xff000050)//blue
	{
		gs_filewriteint(1,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(0,f);
		gs_fileseek(f,-3,1);
	}
}
func exportMap()
{
	
	nameoffs=0;
	mapw = GameGet(G_MAPW) ;
	maph = GameGet(G_MAPH);
	roomw=GameGet(G_ROOMW);
	roomh=GameGet(G_ROOMH);
	print ("mapw=");
	print (mapw);
	print("\n");
	print ("maph=");
	print (maph);
	print("\n");
	print ("roomw=");
	print (roomw);
	print("\n");
	print ("roomh=");
	print (roomh);
	print("\n");
	
	
	filename = "dizzy.0";

	f = gs_fileopen(filename,1);
	rnames = gs_fileopen("roomnames.exp",0);
	
	if(!f) return 0;
	//----------------
	
	rooms = tab(maph);
	orooms = tab(maph);
	items = tab(0);
	itemscount=0;
	
	mat = tab(17);
	shifts=tab(maph*mapw+1);
	buf="";
	buf=(str)mapw+","+(str)maph;
	gs_filewriteint(mapw,f);//ширина карты
	gs_fileseek(f,-3,1);
	gs_filewriteint(maph,f);//высота карты
	gs_fileseek(f,-3,1);
	for(a=0;a<17;a++)
	{
		mat[a]= tab(30);
	}
	for(a=0;a<maph;a++)
	{
		rooms[a] = tab(mapw);
		orooms[a] = tab(mapw);
	}
	for(a=0;a<maph;a++)
	{
		for(b=0;b<mapw;b++)
		{
			rooms[a][b] = tab(0);
			orooms[a][b] = tab(0);
		}
	}
	
	brushcnt=BrushCount();
	print ("total brush count=");
	print (brushcnt);
	print("\n");
	for(a=0;a<brushcnt;a++)
	{
			orx=(int)(BrushGet(a,B_X)/roomw);
			ory=(int)(BrushGet(a,B_Y)/roomh);
			if(orx<mapw && ory <maph)
			{
			sz=tabsize(&rooms[ory][orx]);
			tabadd(&rooms[ory][orx],1);
			rooms[ory][orx][sz]=a;
			}
			
	}
	objcnt=ObjCount();
	print ("total objects count=");
	print (objcnt);
	print("\n");
	for(a=0;a<objcnt;a++)
	{
		if(ObjGet(a,O_CLASS)!=4)//экспортируем всё кроме item'ов. Их экспортируем отдельно
		{
			orx=(int)(ObjGet(a,O_X)/roomw);
			ory=(int)(ObjGet(a,O_Y)/roomh);
			if(orx<mapw && ory <maph)
			{
			sz=tabsize(&orooms[ory][orx]);
			tabadd(&orooms[ory][orx],1);
			orooms[ory][orx][sz]=a;
			}
		} else { //сохраним в отдельный массив id объектов
			
			tabadd(&items,1);
			items[itemscount]=a;
			itemscount++;
		};
	}

	curshift=49152+8+(mapw*maph*2);
	itemsoffs=gs_filetell(f);
	gs_fileseek(f,4,1);//смещения на предметы заглушка
	gs_fileseek(f,2,1);//смещения на названия комнат заглушка
	gs_filewriteint((int)curshift,f);//смещение на первую комнату
	gs_fileseek(f,-2,1);
	print ("offset to room 0,0=");
	print (curshift-49152);
	print("\n");
	for(a=0;a<maph;a++)
	{
		
		for(b=0;b<mapw;b++)
		{
			//curshift=shifts[b+a*b];
			curshift+=8;
			if(tabsize(&rooms[a][b])>0)
			{
				for(c=0;c<tabsize(&rooms[a][b]);c++)
				{
					curshift+=10;
				}
			}
			else
			{
				curshift+=1;
			}
			//curshift+=1;
			if(tabsize(&orooms[a][b])>0)
			{
				for(c=0;c<tabsize(&orooms[a][b]);c++)
				{
					curshift+=26;
				}
			}
			else
			{
				//curshift+=1;
			}
			gs_filewriteint((int)curshift,f);//смещения на комнаты
			gs_fileseek(f,-2,1);
			print ("offset to room ");
			print(a);
			print(",");
			print(b);
			print("=");
			print (curshift-49152);
			print("\n");
		}
	}
	gs_fileseek(f,-2,1);
	//вывод комнат
	//---------------------
	for(a=0;a<maph;a++)
	{
		for(b=0;b<mapw;b++)
		{
			print ("first is offset to name=");
			gs_filereadint(&nameoffs,rnames);
			println (nameoffs);
			
			gs_filewriteint(nameoffs,f);//смещение на название комнаты
			gs_fileseek(f,-2,1);
			if(tabsize(&rooms[a][b])>0)
			{
				print ("brush count in room ");
				print(a);
				print(",");
				print(b);
				print("=");
				print (tabsize(&rooms[a][b]));
				print("\n");
				
				//sorting
				for(c=0;c<tabsize(&rooms[a][b])-1;c++)
				{
					if(BrushGet(rooms[a][b][c],B_LAYER)>BrushGet(rooms[a][b][c+1],B_LAYER))
					{
						bufr=rooms[a][b][c];
						rooms[a][b][c]=rooms[a][b][c+1];
						rooms[a][b][c+1]=bufr;
						c=-1;
					}
				}
				//exporting
				
				
				gs_filewriteint(tabsize(&rooms[a][b]),f);//кол-во статичкских объектов
				gs_fileseek(f,-3,1);
				dynamic=gs_filetell(f);
				gs_fileseek(f,4,1);//смещения на динамические объекты заглушка
				

				
				for(c=0;c<tabsize(&rooms[a][b]);c++)
				{

					orx=(int)(BrushGet(rooms[a][b][c],B_X)/roomw);
					ory=(int)(BrushGet(rooms[a][b][c],B_Y)/roomh);
					
					gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_X)-(orx*roomw))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_Y)-(ory*roomh))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_W))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_H))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_MAP))/8,f);
					gs_fileseek(f,-3,1);
					if (BrushGet(rooms[a][b][c],O_TILE)==511)
					{
						gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_MAP+1))/8,f);
					}
					if (BrushGet(rooms[a][b][c],O_TILE)==512)
					{
						gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_MAP+1))/8+32,f);
					}
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_MAP+2)-BrushGet(rooms[a][b][c],B_MAP))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(BrushGet(rooms[a][b][c],B_MAP+3)-BrushGet(rooms[a][b][c],B_MAP+1))/8,f);
					gs_fileseek(f,-3,1);
					color=BrushGet(rooms[a][b][c],B_COLOR);
					write_color(color,f);
				}
			}
			else
			{
				gs_filewriteint(0,f);
				gs_fileseek(f,-3,1);
				dynamic=gs_filetell(f);
				gs_fileseek(f,4,1);//смещения на динамические объекты заглушка
				gs_filewriteint(0,f);
				gs_fileseek(f,-3,1);
			}
			store=gs_filetell(f);
			gs_fileseek(f,dynamic,0);
			gs_filewriteint(store+49152,f);
			//gs_filewriteint(65535,f);
			
			print ("offset to dynamic objects=");
			print (store);
			print("\n");
			gs_fileseek(f,store,0);
			//--------------------------------------------------------------
			//экспортируем динамические объекты
			//--------------------------------------------------------------
	
			
		

			//sorting
			for(c=0;c<tabsize(&orooms[a][b])-1;c++)
			{
				if((ObjGet(orooms[a][b][c],O_COLLIDER)==1 && ObjGet(orooms[a][b][c+1],O_COLLIDER)==1 && ObjGet(orooms[a][b][c],O_ID)>ObjGet(orooms[a][b][c+1],O_ID))
					||
				(ObjGet(orooms[a][b][c],O_COLLIDER)!=1 && ObjGet(orooms[a][b][c+1],O_COLLIDER)==1))
				{
					
					
					bufr=orooms[a][b][c];
					orooms[a][b][c]=orooms[a][b][c+1];
					orooms[a][b][c+1]=bufr;
					c=-1;
				}
			}
			//exporting
			gs_filewriteint(tabsize(&orooms[a][b]),f);//кол-во динамических объектов
			gs_fileseek(f,-3,1);
			dynamic=gs_filetell(f);

			println("dynamic objects -------------------------");
			print("room y=");
			println(a);
			print("room x=");
			println(b);
			print("objects count =");
			println(tabsize(&orooms[a][b]));
			for(c=0;c<tabsize(&orooms[a][b]);c++)
			{
				sprnum=0;
				if (ObjGet(orooms[a][b][c],O_TILE)==511)
				{
					ObjSet(orooms[a][b][c],O_DRAW,3);
					odraw=3;
				}
				if (ObjGet(orooms[a][b][c],O_TILE)==512)
				{
					ObjSet(orooms[a][b][c],O_DRAW,3);
					odraw=4;
				}
				if (ObjGet(orooms[a][b][c],O_TILE)==513)
				{
					ObjSet(orooms[a][b][c],O_DRAW,2);
					odraw=2;
					x1=ObjGet(orooms[a][b][c],B_MAP);
					y1=ObjGet(orooms[a][b][c],B_MAP+1);
					ff = gs_fileopen("lspriteset2.txt",0);
					
					flag=1;
					strx="";
					stry="";
					while(gs_filereadtext(&strx,ff) && flag==1)
					{
						if(strsub(strx,0,1)=="0")
						{
							strx=strsub(strx,1,1);
						}
						gs_filereadtext(&stry,ff);
						if(strsub(stry,0,1)=="0")
						{
							stry=strsub(stry,1,1);
						}
						
						if(x1==(int)strx && y1==(int)stry)
						{
							flag=0;
							
						}
						else 
						{
							sprnum++;
						}
					}
			
				}					
				
				
				print ("collider=");
				print ((int)(ObjGet(orooms[a][b][c],O_COLLIDER)));
				print("\n");
				print("offset to dinamic object=");
				println(gs_filetell(f));
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_COLLIDER)),f);
				gs_fileseek(f,-3,1);
				print ("disable=");
				print ((int)(ObjGet(orooms[a][b][c],O_DISABLE)));
				print("\n");
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_DISABLE)),f);
				gs_fileseek(f,-3,1);
				print ("id=");
				print ((int)(ObjGet(orooms[a][b][c],O_ID)));
				print("\n");
				println (ObjGet(orooms[a][b][c],O_COLLIDER));
				
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_ID)),f);
				gs_fileseek(f,-3,1);
				
				orx=(int)(ObjGet(orooms[a][b][c],B_X)/roomw);
				
				ory=(int)(ObjGet(orooms[a][b][c],B_Y)/roomh);
				
				if(odraw==3||odraw==4)
				{
					
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_X)-(orx*roomw))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_Y)-(ory*roomh))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_W))/8,f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_H))/8,f);
					gs_fileseek(f,-3,1);
					
				} else {
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_X)-(orx*roomw)),f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_Y)-(ory*roomh)),f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_W)),f);
					gs_fileseek(f,-3,1);
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_H)),f);
					gs_fileseek(f,-3,1);
				}
				
				if(odraw==3||odraw==4)
				{
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_MAP))/8,f);
				}
				else
				{
					gs_filewriteint((int)(sprnum),f);
				}
				gs_fileseek(f,-3,1);
				if(odraw==3||odraw==2)
				{
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_MAP+1))/8,f);
				}
				if(odraw==4)
				{
					gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_MAP+1))/8+32,f);
				}
				
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_MAP+2)-ObjGet(orooms[a][b][c],B_MAP))/8,f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_MAP+3)-ObjGet(orooms[a][b][c],B_MAP+1))/8,f);
				gs_fileseek(f,-3,1);
			
				color=ObjGet(orooms[a][b][c],O_COLOR);
				write_color(color,f);
				if (ObjGet(orooms[a][b][c],O_ANIM)==1)
				{
					while(ObjGet(orooms[a][b][c],O_FRAME)>=ObjGet(orooms[a][b][c],O_TARGET))
					{
						
						ObjSet(orooms[a][b][c],O_FRAME,(ObjGet(orooms[a][b][c],O_FRAME)-ObjGet(orooms[a][b][c],O_TARGET)));
					}
				}
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_FRAME)),f);
				gs_fileseek(f,-3,1);
				print ("FRAME=");
				println(ObjGet(orooms[a][b][c],O_FRAME));
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_ANIM)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_DELAY)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(1),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_TARGET)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_MATERIAL)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_DRAW)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_CLASS)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_STATUS)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_USER)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_USER+1)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_USER+2)),f);
				gs_fileseek(f,-3,1);
				gs_filewriteint((int)(ObjGet(orooms[a][b][c],O_USER+3)),f);
				gs_fileseek(f,-3,1);
				
			}
			
		}
	}
	store=gs_filetell(f);
	gs_fileseek(f,itemsoffs,0);
	gs_filewriteint(store+49152,f);
	//gs_filewriteint(65535,f);
	
	print ("offset to items=");
	print (store);
	print("\n");
	gs_fileseek(f,store,0);
	//--------------------------------------------------------------
	//экспортируем предметы
	//--------------------------------------------------------------
	
	
	//sorting
	for(c=0;c<tabsize(&items)-1;c++)
	{
		if(ObjGet(items[c],O_ID)==1 > ObjGet(items[c+1],O_ID))
		{
			bufr=items[c];
			items[c]=items[c+1];
			items[c+1]=bufr;
			c=-1;
		}
	}
	//exporting
	gs_filewriteint(tabsize(&items),f);//кол-во предметов
	gs_fileseek(f,-3,1);
	println("items -------------------------");
	for(c=0;c<tabsize(&items);c++)
	{
		orx=(int)((ObjGet(items[c],O_X)/roomw));
		ory=(int)((ObjGet(items[c],O_Y)/roomh));
		if(orx>=mapw)orx=0;
		if(ory>=maph)ory=0;
		print(orx);
		print(" X ");
		println(ory);
		gs_filewriteint(orx,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint(ory,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)ObjGet(items[c],O_STATUS),f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)ObjGet(items[c],O_ID),f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)(ObjGet(items[c],O_X)-(orx*roomw))/8,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)(ObjGet(items[c],O_Y)-(ory*roomh))/8,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)(ObjGet(items[c],O_W)/8),f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)(ObjGet(items[c],O_H)/8),f);
		gs_fileseek(f,-3,1);
		
		gs_filewriteint((int)(ObjGet(items[c],O_MAP))/8,f);
		gs_fileseek(f,-3,1);
		if(ObjGet(items[c],O_TILE)==511||ObjGet(items[c],O_TILE)==513)
		{
			gs_filewriteint((int)(ObjGet(items[c],O_MAP+1))/8,f);
		}
		if(ObjGet(items[c],O_TILE)==512)
		{
			gs_filewriteint((int)(ObjGet(items[c],O_MAP+1))/8+32,f);
		}
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)(ObjGet(items[c],O_MAP+2)-ObjGet(items[c],O_MAP))/8,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)(ObjGet(items[c],O_MAP+3)-ObjGet(items[c],O_MAP+1))/8,f);
		gs_fileseek(f,-3,1);
		gs_filewriteint((int)ObjGet(items[c],O_USER),f);
		println("O_USER");
		println((int)ObjGet(items[c],O_USER));
		
		gs_fileseek(f,-3,1);
		color=ObjGet(items[c],O_COLOR);
		write_color(color,f);
		println(gs_filetell(f));
		
		
	}
	gs_fileclose(f);
	
	gs_fileclose(rnames);
	
}
func BeginNewGame()
{
	GameSet(G_PAUSE,0);							// unpause the game
	PlayerSet(P_DISABLE,0);						// enable player
	PlayerSetPos(PLAYER_BEGINX,PLAYER_BEGINY);	// set begin position
	MusicFade(0,1);								// set music fade options
	MusicPlay(MUSIC_DEFAULT);					// play default music
	exportMap();
	upload_materials();
}
	

	//---------------------


/////////////////////////////////////////////////////////////////////////////////
// Interactions
/////////////////////////////////////////////////////////////////////////////////

//...

/////////////////////////////////////////////////////////////////////////////////
